@@ -74,19 +74,69 @@
 ################################################################################
 # Check the parameters
 ################################################################################
-.checkParameter <- function(parameter){
+.checkParameter <- function(parameter, mlxProject=F){
   if(!is.null(parameter)){
-    if(!(is.vector(parameter)||(is.data.frame(parameter))))
-      stop("Invalid paramerer. It must be a vector or a data.frame.", call. = F)
+    # Monolix parameters
+    if (is.string(parameter)) {
+      if (parameter %in% c("mode", "mean", "pop")) {
+        message("[INFO] 'mode', 'mean', 'pop' are deprecated parameters. Parameters imported from",
+                " monolix must be in {'", paste0(.getAllowedMlxParameterNames(), collapse="', '"),
+                "'}.")
+        if (parameter == "mode") {
+          parameter <- "mlx_EBEs"
+        } else if (parameter == "mean") {
+          parameter <- "mlx_CondMean"
+        }
+      }
+      # parameter path to dataframe
+      if (file.exists(parameter)) {
+        .checkExtension(.getFileExt(parameter), "parameter file extension")
+      } else if (parameter %in% .getAllowedMlxParameterNames()) {
+        if (! mlxProject) {
+          stop("Invalid parameter. Monolix parameters can only be used when a project is imported from monolix.", call. = F)
+        }
+      } else {
+        stop("Invalid parameter. Parameters defined as string must be either a path to a csv or txt file or ",
+              "a parameter imported from monolix ('",
+              paste0(.getAllowedMlxParameterNames(), collapse="', '"), "').",
+             call.=F)
+      }
+    } else {
+      if (!(is.vector(parameter) || (is.data.frame(parameter)))) {
+        stop("Invalid paramerer. It must be a vector or a data.frame.", call. = F)
+      }
+    }
   }
-  # if (is.vector(parameter)) {
-  #   if (! all(sapply(parameter, function(p) length(p) == 1))) {
-  #     stop("Invalid parameter. When parameter is defined as a vector, each defined parameter must be of size 1.", call. = F)
-  #   }
-  # }
   return(parameter)
 }
 
+.checkCovariate <- function(covariate, mlxProject=F){
+  if(!is.null(covariate)){
+    # Monolix covariates
+    if (is.string(covariate)) {
+      # parameter path to dataframe
+      if (file.exists(covariate)) {
+        .checkExtension(.getFileExt(covariate), "covariate file extension")
+
+      } else if (covariate %in% .getAllowedMlxCovariateNames()) {
+        if (! mlxProject) {
+          stop("Invalid covariate. Monolix covariates can only be used when a project is imported from monolix.", call. = F)
+        }
+      } else {
+        stop("Invalid covariate. Covariates defined as string must be either a path to a csv or txt file or ",
+             "a covariate imported from monolix ('",
+             paste0(.getAllowedMlxCovariateNames(), collapse="', '"), "').",
+             call.=F)
+      }
+    } else {
+      if(!(is.vector(covariate)||(is.data.frame(covariate)))) {
+        stop("Invalid covariate It must be a vector or a data.frame.", call. = F)
+      }
+    }
+  }
+  return(covariate)
+}
+
 .checkMissingParameters <- function(parameter, expectedParameters, frommlx = FALSE) {
   diff <- setdiff(expectedParameters, parameter)
   ismissing <- FALSE
@@ -132,7 +182,7 @@
 # - all the mandatoryNames are defined
 # - all the named used in the inputList are in the mandatoryNames or defaultNames
 ################################################################################
-.checkUnitaryList <- function(inputList, mandatoryNames, defaultNames, listName){
+.checkUnitaryList <- function(inputList, mandatoryNames=c(), defaultNames=c(), listName="list"){
   if (is.null(inputList)) {
     return(inputList)
   }
@@ -145,8 +195,6 @@
   # Check that all the defined elements are vectors
   namesList <- names(inputList)
   for (indexName in seq_along(namesList)) {
-    # if (is.list(inputList[[indexName]]) || ! is.vector(inputList[[indexName]]))
-    #   stop("Invalid field ", namesList[indexName], " in `", listName, "`. It must be a vector.", call. = F)
     if (! (is.list(inputList[[indexName]]) || is.vector(inputList[[indexName]]) || is.factor(inputList[[indexName]]))) {
       stop("Invalid field ", namesList[indexName], " in '", listName, "'. It must be a vector.", call. = F)
     }
@@ -173,7 +221,7 @@
 
   # Check that all the elements in the list are either in the mandatory or default list
   extraName <- setdiff(namesList, union(mandatoryNames,defaultNames))
-  if (length(extraName) > 0) {
+  if (!is.null(inputList) && length(extraName) > 0) {
     message <- paste0("Invalid fields will be ignored in '", listName, "'.")
     if( length(extraName) == 1){
       message <- paste0(message, " ", extraName," will be ignored.")
@@ -200,13 +248,16 @@
 ###################################################################################
 .checkUnitaryTreatment<- function(treatment, listName = "treatment"){
   mandatoryNames <- c("time", "amount")
-  defaultNames <- c("tinf", "rate", "type")
+  defaultNames <- c("tinf", "rate", "type", "repeats", "probaMissDose", "washout")
   if (is.data.frame(treatment)) {
     # add id as mandatory name if data is a dataframe
     indexID <- which(names(treatment) == 'id')
     if (length(indexID)) mandatoryNames <- c(mandatoryNames, names(treatment)[indexID])
+    # remove repeats and probaMissDose as default name if data is a dataframe
+    # defaultNames <- setdiff(defaultNames, c("repeats"))
+    defaultNames <- names(treatment)
   }
-  
+
   # replace adm name by type
   names(treatment)[names(treatment) == "adm"] <- "type"
   
@@ -215,7 +266,7 @@
   
   # error if target in treatment
   if (is.element("target", names(treatment))) {
-    stop("Invalid field 'target' for 'treatment' argument. You must use 'adm=' instead.", call. = FALSE)
+    stop("Invalid ", listName, " target. You must use 'adm=' instead.", call. = FALSE)
   }
 
   # check names in treatement
@@ -229,22 +280,22 @@
   if (!is.null(treatment)) {
     # check if the type is an integer
     if (is.element("type", names(treatment))) {
-      .check_strict_pos_integer(treatment$type, "treatment type")
+      .check_strict_pos_integer(treatment$type, paste(listName, "type"))
     } else{
       treatment$type = 1
     }
     
     # check if treatment times are double
-    .check_vector_of_double(treatment$time, "treatment times")
+    .check_vector_of_double(treatment$time, paste(listName, "time"))
     nbTimePoint = length(treatment$time)
     
     # check if treatment amounts are double
-    .check_vector_of_double(treatment$amount, "treatment amount")
+    .check_vector_of_double(treatment$amount, paste(listName, "amount"))
   
     # check that treatment amount vector and treatment times vector have the same length
     # (when length of treatment amount is different from 1)
     if (length(treatment$amount) > 1) {
-      .check_vectors_length(treatment$amount, treatment$time, "treatment amount", "treatment times")
+      .check_vectors_length(treatment$amount, treatment$time, paste(listName, "amount"), paste(listName, "time"))
     } else {
       treatment$amount <- rep(treatment$amount, nbTimePoint)
     }
@@ -252,30 +303,63 @@
     # check that treatment type vector and treatment times vector have the same length
     # (when length of treatment type is different from 1)
     if (length(treatment$type) > 1) {
-      .check_vectors_length(treatment$type, treatment$time, "treatment type", "treatment times")
+      .check_vectors_length(treatment$type, treatment$time, paste(listName, "type"), paste(listName, "time"))
     }
     
     # check that when it exists, treatment rate is a vector of strictly positive double
     if (is.element("rate", names(treatment))) {
-      .check_vector_of_pos_double(treatment$rate, "treatment rate")
+      .check_vector_of_pos_double(treatment$rate, paste(listName, "rate"))
   
       if (length(treatment$rate) > 1) {
-        .check_vectors_length(treatment$rate, treatment$time, "treatment rate", "treatment times")
+        .check_vectors_length(treatment$rate, treatment$time, paste(listName, "rate"), paste(listName, "time"))
       } else {
         treatment$rate <- rep(treatment$rate, nbTimePoint)
       }
     }
     
+    # check that when it exists, treatment washout is a vector of strictly positive double
+    if (is.element("washout", names(treatment))) {
+      .check_vector_of_bool(treatment$washout, paste(listName, "washout"))
+      
+      if (length(treatment$washout) > 1) {
+        .check_vectors_length(treatment$washout, treatment$time, paste(listName, "washout"), paste(listName, "time"))
+      } else {
+        treatment$washout <- rep(treatment$washout, nbTimePoint)
+      }
+      
+      if (is.data.frame(treatment)) {
+        treatment$washout <- as.integer(treatment$washout)
+      } else {
+        treatment$washout <- as.logical(treatment$washout)
+      }
+    }
+    
     # check that when it exists, treatment infusion duration is a vector of positive double
     if (is.element("tinf", names(treatment))) {
-      .check_vector_of_pos_double(treatment$tinf, "treatment infusion duration")
+      .check_vector_of_pos_double(treatment$tinf, paste(listName, "infusion duration"))
   
       if (length(treatment$tinf) > 1) {
-        .check_vectors_length(treatment$tinf, treatment$time, "treatment infusion duration", "treatment times")
+        .check_vectors_length(treatment$tinf, treatment$time, paste(listName, "infusion duration"), paste(listName, "time"))
       } else{
         treatment$tinf <- rep(treatment$tinf, nbTimePoint)
       }
     }
+    
+    # check that when it exists, probaMissDose is a double in [0, 1]
+    if (is.element("probaMissDose", names(treatment))) {
+      .check_double(treatment$probaMissDose, paste(listName, "probaMissDose"))
+      .check_in_range(treatment$probaMissDose, paste(listName, "probaMissDose"), 0, 1)
+    }
+    
+    # check that when it exists, repeats is a vector with cycleDuration and NumberOfRepetitions
+    if (is.element("repeats", names(treatment)) && ! is.data.frame(treatment)) {
+      if (! all(c("cycleDuration", "NumberOfRepetitions") %in% names(treatment$repeats))) {
+        stop("Invalid ", paste(listName, "repeats"), ". It should be a vector specifiying fields cycleDuration and NumberOfRepetition.", call.=F)
+      }
+      .check_pos_integer(treatment$repeats[["NumberOfRepetitions"]], paste(listName, "repeats, NumberOfRepetitions"))
+      .check_pos_double(treatment$repeats[["cycleDuration"]], paste(listName, "repeats, cycleDuration"))
+      
+    }
   } else {
     treatment <- as.list(treatment)
   }
@@ -290,25 +374,88 @@
 # - infusion time is removed if the value is 0
 # - infusion time are positive
 ################################################################################
-.checkTreatment <- function(treatment){
+.checkTreatment <- function(treatment, mlxProject=F){
   # When only one treatment --> create a list of treatment
-  if (is.element("time", names(treatment))) {
+  if (is.string(treatment) | is.element("time", names(treatment))) {
     treatment <- list(treatment)
   }
-  
   for (itreat in seq_along(treatment)) {
     treatementValue <- treatment[[itreat]]
+
     if (length(treatment) > 1) {
-      listName <- paste0("treatment ", itreat)
+      treatName <- paste0("treatment ", itreat)
     } else {
-      listName <- "treatment"
+      treatName <- "treatment"
+    }
+
+    if (is.string(treatementValue)) {
+      # parameter path to dataframe
+      if (file.exists(treatementValue)) {
+        .checkExtension(.getFileExt(treatementValue), paste(treatName, "file extension"))
+        treatementValue <- utils::read.table(file=treatementValue, header=T, sep=.getDelimiter(treatementValue))
+
+      } else if (grepl("^mlx_Adm[0-9]", treatementValue)) {
+        if (! mlxProject) {
+          stop("Invalid ", treatName, ". Monolix treatment can only be used when a project is imported from monolix.", call. = F)
+        }
+      } else {
+        stop("Invalid ", treatName, ". Treatments defined as string must be either a path to a csv or txt file or ",
+             "a treatment imported from monolix with the form 'mlx_Adm'.",
+             call.=F)
+      }
+    }
+    
+    if (! is.string(treatementValue)) {
+      treatment[[itreat]] <- .checkUnitaryTreatment(treatementValue, listName=treatName)
     }
-    treatment[[itreat]] <- .checkUnitaryTreatment(treatementValue, listName = listName)
   }
   treatment <- treatment[sapply(treatment, function(e) length(e) > 0)]
   return(treatment)
 }
 
+#*******************************************************************************
+# OCCASION MANAGEMENT
+#*******************************************************************************
+.checkOccasion <- function(occasion, occName="occasion"){
+  if (is.null(occasion)) return(occasion)
+
+  if (is.string(occasion)) {
+    if (occasion == "none") return(occasion)
+
+    # occasion path to dataframe
+    if (file.exists(occasion)) {
+      .checkExtension(.getFileExt(occasion), paste(occName, "file extension"))
+      occasion <- utils::read.table(file=occasion, header=T, sep=.getDelimiter(occasion))
+    } else {
+      stop("Invalid ", occName, ". Occasions defined as string must be valid path to a csv or txt file.",
+           call.=F)
+    }
+  }
+  if (is.data.frame(occasion)) {
+    if (! is.element("time", names(occasion))) {
+      stop("When occasion is a dataframe, it must contain a time column.", call. = FALSE)
+    }
+    .check_vector_of_double(occasion$time, paste(occName, "time"))
+  } else {
+    occasion <- .checkUnitaryList(
+      occasion,
+      mandatoryNames=c("name", "time"),
+      listName=occName
+    )
+    if (is.null(occasion)) return(occasion)
+
+    occname <- occasion$name
+    .check_char(occname, paste(occName, "name"))
+    if (length(occname) > 1) {
+      stop("Invalid occasion. If you want to define several occasions, use a dataframe.", call.=F)
+    }
+    .check_vector_of_double(occasion$time, paste(occName, "time"))
+    occasion <- data.frame(time=occasion$time)
+    occasion[occname] <- 1:length(occasion$time)
+  }
+  return(occasion)
+}
+
 #*****************************************************************************************************************************************
 # OUTPUT MANAGEMENT
 #*****************************************************************************************************************************************
@@ -322,40 +469,42 @@
   mandatoryNames <- c("name", "time")
   defaultNames <- c("lloq", "uloq", "limit")
   if (!is.null(output)) {
-    if (is.data.frame(output)) {
-      # add id as mandatory name if data is a dataframe
-      indexID <- which(names(output) == 'id')
-      if (length(indexID)) defaultNames <- c(defaultNames, names(output)[indexID], names(output))
-    }
     output <- .checkUnitaryList(
       output,
       mandatoryNames = mandatoryNames,
       defaultNames = defaultNames,
       listName = listName
     )
+    if (is.null(output)) return(as.list(output))
 
     # check that output names are string
-    if (is.element("name", names(output)))
-      .check_vector_of_char(output$name, "output name")
+    .check_vector_of_char(output$name, paste(listName, "name"))
 
-    # check that output times are double
-    if (is.element("time", names(output))) {
-      if (is.data.frame(output$time)) {
-        if (! is.element("time", names(output$time))) {
-          stop("When output time is a dataframe, it must contains at least a time column.", call. = FALSE)
-        }
+    # check that output times are a string, a dataframe, or a double
+    outputTime <- output$time
+    if (is.string(outputTime)) {
+      # parameter path to dataframe
+      if (file.exists(outputTime)) {
+        .checkExtension(.getFileExt(outputTime), paste(listName, "file extension"))
+        outputTime <- utils::read.table(file=outputTime, header=T, sep=.getDelimiter(outputTime))
       } else {
-        if (length(output$time) > 1) {
-          .check_vector_of_double(output$time, "output time")
-        } else {
-          if (output$time != "none") .check_vector_of_double(output$time, "output time") 
-        }
+        stop("Invalid ", listName, ". Outputs defined as string must be valid path to a csv or txt file.",
+             call.=F)
       }
     }
-    # } else {
-    #   output <- NULL
-    #   #output$time = 0
-    # }
+    if (is.data.frame(outputTime)) {
+      if (! is.element("time", names(outputTime))) {
+        stop("When output time is a dataframe, it must contain at least a time column.", call. = FALSE)
+      }
+      .check_vector_of_double(outputTime$time, paste(listName, "time"))
+    } else {
+      if (length(outputTime) > 1) {
+        .check_vector_of_double(outputTime, paste(listName, "time"))
+      } else {
+        .check_vector_of_double(outputTime, paste(listName, "time"))
+      }
+    }
+    output$time <- outputTime
   }
   if (is.null(output)) {
     output <- as.list(output)
@@ -368,19 +517,32 @@
 # - have string for names
 # - have double for times
 ################################################################################
-.checkOutput <- function(output){
+.checkOutput <- function(output, mlxProject=F){
   # When only one output --> create a list of output
-  if (is.data.frame(output) | is.element("name", names(output))) {
+  if (is.string(output) || is.element("name", names(output))) {
     output <- list(output)
   }
+  
   for (iout in seq_along(output)) {
     outputValue <- output[[iout]]
     if (length(output) > 1) {
-      listName <- paste0("output ", iout)
+      outName <- paste0("output ", iout)
     } else {
-      listName <- "output"
+      outName <- "output"
+    }
+    
+    if (is.string(outputValue)) {
+      if (grepl("^mlx_", outputValue)) {
+        if (! mlxProject) {
+          stop("Invalid ", outName, ". Monolix output can only be used when a project is imported from monolix.", call. = F)
+        }
+      } else {
+        stop("Invalid ", outName, ". Outputs defined as string must be ",
+             "an output imported from monolix with the form 'mlx_'.", call.=F)
+      }
+    } else {
+      output[[iout]] <- .checkUnitaryOutput(outputValue, listName = outName)
     }
-    output[[iout]] <- .checkUnitaryOutput(outputValue, listName = listName)
   }
   output <- output[sapply(output, function(e) length(e) > 0)]
   if (!length(output)) output <- NULL
@@ -452,22 +614,27 @@
     defaultNames = defaultNames,
     listName = listName
   )
+  if (is.null(regressor)) return(as.list(regressor))
   
   # check that regressor name is a string
   if (is.element("name", names(regressor))) {
     if (length(regressor$name) > 1)
       stop("The regressor name must have only one value.", call. = F)
-    .check_char(regressor$name, "regressor name")
+    .check_char(regressor$name, paste(listName, "name"))
   }
 
   # check that regressor values are double
   if(is.element("value", names(regressor))) {
-    .check_vector_of_double(regressor$value, "regressor value")
+    .check_vector_of_double(regressor$value, paste(listName, "value"))
   }
 
   # check that regressor times are double
   if (is.element("time", names(regressor))) {
-    .check_vector_of_double(regressor$time, "regressor time")
+    .check_vector_of_double(regressor$time, paste(listName, "time"))
+  }
+  
+  if (all(c("time", "value") %in% names(regressor))) {
+    .check_vectors_length(regressor$time, regressor$value, paste(listName, "time"), paste(listName, "value"))
   }
 
   if (is.null(regressor)) regressor <- as.list(regressor)
@@ -482,7 +649,7 @@
 ################################################################################
 .checkRegressor <- function(regressor){
   # When only one regressor --> create a list of regressor
-  if (is.data.frame(regressor) | is.element("name", names(regressor))) {
+  if (is.string(regressor) | is.data.frame(regressor) | is.element("name", names(regressor))) {
     regressor <- list(regressor)
   }
   for (ireg in seq_along(regressor)) {
@@ -492,7 +659,18 @@
     } else {
       listName <- "regressor"
     }
-    regressor[[ireg]] <- .checkUnitaryRegressor(regValue, listName = listName)
+
+    if (is.string(regValue)) {
+      # regressor path to dataframe
+      if (file.exists(regValue)) {
+        .checkExtension(.getFileExt(regValue), paste(listName, "file extension"))
+        regValue <- utils::read.table(file=regValue, header=T, sep=.getDelimiter(regValue))
+      } else {
+        stop("Invalid ", listName, ". Regressors defined as string must be valid path to a csv or txt file.",
+             call.=F)
+      }
+    }
+    regressor[[ireg]] <- .checkUnitaryRegressor(regValue, listName=listName)
   }
   regressor <- regressor[sapply(regressor, function(e) length(e) > 0)]
   return(regressor)
@@ -510,38 +688,91 @@
 # - treatment is a valid treatment,
 # - regressor is a valid regressor
 ################################################################################
-.checkUnitaryGroup<- function(group, listName){
+.checkUnitaryGroup <- function(group, listName, mlxProject=F){
+  nbID <- 1
+
   # check if the size is an integer
   if (is.element("size", names(group))) {
     if (length(group$size) > 1) {
       stop("group size must a strictly positive integer of size 1", call. = F)
     }
     .check_strict_pos_integer(group$size, "treatment size")
-  } else {
-    #group$size = 1
   }
   
   # check that parameter is a good parameter
   if (is.element("parameter", names(group))) {
-    group$parameter <- .transformParameter(parameter=group$parameter)
-    group$parameter <- .checkParameter(parameter=group$parameter)
+    group$parameter <- .transformParameter(group$parameter)
+    group$parameter <- .checkParameter(parameter=group$parameter, mlxProject=mlxProject)
+    # Write the data set as a .txt file
+    if (is.data.frame(group$parameter)) {
+      group$parameter <- .addDataFrameTemp(df=group$parameter)
+    }
+    # update the size of the default group
+    if (is.string(group$parameter) && file.exists(group$parameter)) {
+      df <- utils::read.table(file=group$parameter, header=T, sep=.getDelimiter(group$parameter))
+      nbID <- max(nbID, .getNbIds(df))
+    }
+  }
+
+  # check that covariate is a good covariate
+  if (is.element("covariate", names(group))) {
+    group$covariate <- .transformParameter(group$covariate)
+    group$covariate <- .checkCovariate(covariate=group$covariate, mlxProject=mlxProject)
+    # Write the data set as a .txt file
+    if (is.data.frame(group$covariate)) {
+      group$covariate <- .addDataFrameTemp(df=group$covariate)
+    }
+    # update the size of the default group
+    if (is.string(group$covariate) && file.exists(group$covariate)) {
+      df <- utils::read.table(file=group$covariate, header=T, sep=.getDelimiter(group$covariate))
+      nbID <- max(nbID, .getNbIds(df))
+    }
   }
 
   # check that output is a good output
   if (is.element("output", names(group))) {
-    group$output <- .checkOutput(output = group$output)
+    group$output <- .checkOutput(output = group$output, mlxProject=mlxProject)
+    for (iout in seq_along(group$output)) {
+      outputValue <- group$output[[iout]]
+      if (is.data.frame(outputValue$time)) {
+        dfout <- outputValue$time
+        nbID <- max(nbID, .getNbIds(dfout))
+        # Write the data set as a .txt file
+        group$output[[iout]]$time <- .addDataFrameTemp(df = dfout)
+      }
+    }
   }
 
   # check that treatment is a good treatment
   if (is.element("treatment", names(group))) {
-    group$treatment <- .checkTreatment(treatment = group$treatment)
+    group$treatment <- .checkTreatment(treatment=group$treatment, mlxProject=mlxProject)
     group$treatment <- .splitTreatment(group$treatment)
+
+    # if dataframe, save it in a txt file
+    for (itreat in seq_along(group$treatment)) {
+      treatementValue <- group$treatment[[itreat]]
+      if (is.data.frame(treatementValue)) {
+        # Write the data set as a .txt file
+        nbID <- max(nbID, .getNbIds(treatementValue))
+        group$treatment[[itreat]] <-.addDataFrameTemp(df = treatementValue)
+      }
+    }
   }
   
   # check that regressor is a good regressor
   if (is.element("regressor", names(group))) {
     group$regressor <- .checkRegressor(regressor = group$regressor)
     group$regressor <- .transformRegressor(group$regressor)
+    # When regressor is defined as dataframe, write the data set as a .txt file
+    if (is.data.frame(group$regressor)) {
+      nbID <- max(nbID, .getNbIds(group$regressor))
+      group$regressor <- .addDataFrameTemp(df = group$regressor)
+    }
+  }
+  
+  group$nbID <- nbID
+  if (is.null(group$size) && nbID > 1) {
+    group$size <- nbID
   }
   return(group)
 }
@@ -554,12 +785,23 @@
 # - treatment is a valid treatment,
 # - regressor is a valid regressor
 ################################################################################
-.checkGroup <- function(group){
+.checkGroup <- function(group, defaultArguments, mlxProject=F){
   # When only one group --> create a list of group
-  allowedNames <- c("size", "parameter", "output", "treatment", "regressor", "level")
+  allowedNames <- c("size", "parameter", "covariate", "output", "treatment", "regressor", "level")
   if (any(is.element(allowedNames, names(group))) | !all(sapply(group, .is_list_or_named_vector))) {
     group <- list(group)
   }
+  
+  # Check if elements defined in main and groups
+  for (element in c("parameter", "covariate", "output", "treatment", "regressor")) {
+    elementInGroup <- any(sapply(group, function(g) element %in% names(g)))
+    
+    if (elementInGroup & defaultArguments[[element]]) {
+      warning(element, " has been defined in both shared group and groups. ",
+              element, " defined in shared group will be erased.", call.=F)
+    }
+  }
+  
   for (igroup in seq_along(group)) {
     gValue <- group[[igroup]]
     if (length(group) > 1) {
@@ -567,7 +809,7 @@
     } else {
       listName <- "group"
     }
-    group[[igroup]] <- .checkUnitaryGroup(gValue, listName = listName)
+    group[[igroup]] <- .checkUnitaryGroup(gValue, listName=listName, mlxProject=mlxProject)
   }
   return(group)
 }
@@ -641,7 +883,7 @@
     
 }
 
-# Check simpop corr argument ----------------------------------------------
+# Check simpop corr argument ---------------------------------------------------
 .checkSimpopCorr <- function(corr, nbParams) {
   # corr matrix size must be (nbParams x nbParams)
   # if (!is.matrix(corr) | !is.data.frame(corr))
@@ -653,9 +895,61 @@
   return(corr)
 }
 
+# Check sharedIds argument -----------------------------------------------------
+.checkSharedIds <- function(sharedIds, sharedElements) {
+  for (element in sharedIds) {
+    if (element != "output") {
+      if (! any(sapply(sharedElements[[element]], is.string))) {
+        warning("No id to share in ", element, " element.", call.=F)
+        sharedIds <- sharedIds[sharedIds != element]
+      }
+    } else {
+      if (! any(sapply(sharedElements[[element]], function(x) is.string(x$time)))) {
+        warning("No id to share in ", element, " element.", call.=F)
+        sharedIds <- sharedIds[sharedIds != element]
+      }
+    }
+  }
+  if (length(sharedIds) == 0) sharedIds <- NULL
+  return(sharedIds)
+}
+
+# Check new additional line to add to the model --------------------------------
+.checkAdditionalLine <- function(line) {
+  if (is.element("section", names(line))) {
+    if (! grepl("LONGITUDINAL", line$section))
+      stop("'addlines' argument only support 'LONGITUDINAL' section and 'EQUATION' block.", call. = FALSE)
+  }
+  if (is.element("block", names(line))) {
+    if (grepl("EQUATION", line))
+      stop("'addlines' argument only support 'LONGITUDINAL' section and 'EQUATION' block.", call. = FALSE)
+  }
+  return(invisible(TRUE))
+}
+
 ################################################################################
 # Object types check functions
 ################################################################################
+# Check project extension & add extension if needed
+.checkProjectExtension <- function(project_file, software) {
+  ext <- .getFileExt(project_file)
+  if (is.null(ext)) {
+    if (software == "simulx") {
+      project <- paste0(project, ".smlx")
+    } else if (software == "monolix") {
+      project <- paste0(project, ".mlxtran")
+    } else {
+      return()
+    }
+  } else {
+    if (software == "simulx" & ext != "smlx")
+      stop("Invalid extension for a simulx project. smlx extension is expected.", call. = F)
+    if (software == "monolix" & ext != "mlxtran")
+      stop("Invalid extension for a Monolix project. mlxtran extension is expected.", call. = F)
+  }
+  return(project_file)
+}
+
 # Checks if a file exists and returns an error----------------------------------
 .check_file <- function(filename, fileType = "File") {
   if (!file.exists(filename))
@@ -752,10 +1046,17 @@
 # Checks if an input is boolean  -----------------------------------------------
 .check_bool <- function(bool, argname) {
   if (!is.logical(bool))
-    stop("Invalid ", argname, ". It must be logical", call. = F)
+    stop("Invalid ", argname, ". It must be logical.", call. = F)
   return(bool)
 }
 
+.check_vector_of_bool <- function(v, argname) {
+  if (! (is.logical(v) || all(v %in% c(0, 1)))) {
+    stop("Invalid ", argname, ". It must be a vector of boolean.", call. = F)
+  }
+  return(v)
+}
+
 # check if 2 vectors have the same size
 .check_vectors_length <- function(v1, v2, argname1, argname2) {
   if (length(v1) != length(v2))
